open Unix;;

module type S =
  sig
    val addr_number : int
    val total : float ref
    val gethostbyname : string -> Unix.host_entry
    val check : unit -> unit
    val reset : unit -> unit
  end

module Make(P: sig val addr_number : int val not_found : int end)  = struct 

let addr_number = P.addr_number;;

let _ = Random.init (int_of_float (Unix.time()));;

type entry = 
    { delay : float; 
      result : host_entry option; 
      lock : Mutex.t; mutable access : int};; 

let inet_addr_of_int i =
  let a0 = string_of_int ((i/0xffffff) mod 0xff)
  and a1 = string_of_int ((i/0xffff) mod 0xff)
  and a2 = string_of_int ((i/0xff) mod 0xff)
  and a3 = string_of_int (i mod 0xff) in
  let addr = String.concat "." [ a0; a1; a2; a3 ] in
  inet_addr_of_string addr;;

let total = ref 0.;;

let init_entry i =
  let d =  Random.float 0.1 in
  total := !total +. d;
  { delay = d;
    result =
    if i mod P.not_found = 0 then
      None
    else
      Some { h_name = (string_of_int i);
             h_aliases = Array.make 0 "";
             h_addrtype = PF_INET;
             h_addr_list = Array.make 1 (inet_addr_of_int i) };
    lock = Mutex.create();
    access = 0 };;
       
let table = Array.init addr_number init_entry;;

let gethostbyname name =
  let entry = table.(int_of_string name) in
  Thread.delay entry.delay;
  Mutex.lock entry.lock;
  entry.access <- entry.access + 1;
  Mutex.unlock entry.lock;
  match entry.result with
    Some value -> value
  | None -> raise Not_found;;

let check () =
  for i = 0 to (Array.length table) - 1 do
    if table.(i).access <> 1 then
      if table.(i).access = 0 then
        failwith (string_of_int i ^ ": No access.")
      else
        failwith (string_of_int i ^ ": Multiple access")
  done;;

let reset () =
  for i = 0 to (Array.length table) - 1 do
    table.(i).access <- 0
  done;;

end

